# Utility scripts used in the book

proc print_args {args} {
    puts "Args: [join $args {, }]"
}

proc print_list {l} {
    puts $::stdout_chan [join $l \n]
}

proc print_sorted {l} {
    print_list [lsort -dictionary $l]
}

# From the tcllib debug package
proc print_dict {dict args} {
    if {[llength $args] == 0} {
        set names [lsort -dict [dict keys $dict]]
    } else {
        set names {}
        foreach pattern $args {
            # Only sort within each pattern
            lappend names {*}[lsort -dict [dict keys $dict $pattern]]
        }
    }
    set maxl 0
    foreach name $names {
        if {[string length $name] > $maxl} {
            set maxl [string length $name]
        }
    }
    set maxl [expr {$maxl + 2}]
    set lines {}
    foreach name $names {
        set nameString [format %s $name]
        lappend lines [format "%-*s = %s"  $maxl $nameString  [dict get $dict $name]]
    }
    puts $::stdout_chan [join $lines \n]
}

proc print_array {args} {
    uplevel 1 parray $args
}

proc wait {ms} {
    after $ms [list set ::_wait_flag 1]
    vwait ::_wait_flag
}

proc lambda {params body args} {
    return [list ::apply [list $params $body] {*}$args]
}

proc bin2hex {args} {
    regexp -inline -all .. [binary encode hex [join $args ""]]
}
 
if {![catch {package require twapi_com}]} {
    # Only if twapi_com is available
    if {[llength [info commands twapi::comobj_destroy]] == 0} {
        # 4.0 TWAPI does not comobj_destroy
        proc twapi::comobj_destroy args {
            foreach obj $args {
                catch {$obj -destroy}
            }
        }
    }
}
