# Here is an example server that services multiple clients.
# It assumes a line oriented protocol and responds to each line from
# a client with the client's port number followed by the received
# line with the characters reversed.

# We start off by defining the command that will invoked in
# response to every connection request. We want to be able to serve
# multiple clients so we make the accepted client channel non-blocking
# and attach a read handler.

proc on_accept {so client_ip client_port} {
    chan configure $so -buffering line -encoding utf-8 -blocking 0 -translation crlf
    chan event $so readable [list on_read $so $client_port]
}

# Our read handler reverses the line and sends it back to the client.
# As a special case, an empty line from the client will end our
# server.

proc on_read {so client_port} {
    set n [gets $so line]
    if {$n > 0} {
        puts $so "$client_port: [string reverse $line]"
        return
    } elseif {$n == 0} {
        exit 0;                 # Treat empty line as exit server command (*any* client can shut down server)
    } elseif {[eof $so]} {
        close $so
    }
}

# Now we create the listening socket on a port. Instead of picking a
# port ourselves, we will let the system pick one for us by specifying
# `0` as the port number. Also we will only listen on our local
# loopback address. After creating the listening socket
# the server will sit waiting in the event loop.
set listener [socket -server on_accept -myaddr 127.0.0.1 49999]
vwait forever

